/************************************************************************
 *                                                                      *
 * dragAndDrop.c -- CDE drag and drop functions to provide              *
 *                  basic support for file drops.                       *
 *                                                                      *
 * Written By Fredrik Jnsson                                           *
 *                                                                      *
 ************************************************************************/
/* $Id: dragAndDrop.c,v 1.6 1997/10/06 15:53:03 fjo Exp $ */

#include "dragAndDrop.h"

/*
 * fileTransferCallback -- Called by neditTransferCallback if the dropaction
 *						   was a filedrop.
 */
void fileTransferCallback (Widget	widget,
	        	   XtPointer	clientData,
			   XtPointer	callData) {

    DtDndTransferCallbackStruct *transferInfo =
    	(DtDndTransferCallbackStruct *) callData;
    char	*filePath, *name, *path;
    int     ii;

    if (transferInfo == NULL) {
	return;
    }

    /*
     * Verify the protocol and callback reasons
     */
    if (transferInfo->dropData->protocol != DtDND_FILENAME_TRANSFER ||
	transferInfo->reason != DtCR_DND_TRANSFER_DATA) {
	return;
    }
       
    /*
     * Open the file(s). 
     */
    for (ii = 0; ii < transferInfo->dropData->numItems; ii++) {
	filePath = transferInfo->dropData->data.files[ii];
     	if ((name = strrchr(filePath,'/')) == NULL) {
	    name = filePath;
            path = NULL;
        } else {
       	    name++;
       	    path = (char *)strdup(filePath);
     	    strrchr(path,'/')[1] = '\0';
       	} 
	EditExistingFile (WidgetToWindow (widget), name, path, 0);
	if (path != NULL)
	    free(path);
    }
}


/*
 * dataTransferCallback -- Called by neditTransferCallback if the dropaction
 *		           was a data drop.
 */
void dataTransferCallback (Widget	widget,
	        	   XtPointer	clientData,
			   XtPointer	callData) {
	
    DtDndTransferCallbackStruct *transferInfo =
	(DtDndTransferCallbackStruct *) callData;
    int     ii;

    if (transferInfo == NULL) {
	return;
    }

    /*
     * Verify the protocol and callback reasons
     */
    if (transferInfo->dropData->protocol != DtDND_BUFFER_TRANSFER ||
	transferInfo->reason != DtCR_DND_TRANSFER_DATA) {
	return;
    }
	       
    for (ii = 0; ii < transferInfo->dropData->numItems; ii++) {
  	WindowInfo *window, *w;
    	int i;
    	char name[20];
		
	window = WidgetToWindow(widget);

	/* Find a (relatively) unique name for the new file */
	for (i=0; i<INT_MAX; i++) {
    	    if (i == 0)
            	sprintf(name, "Untitled");
            else
            	sprintf(name, "Untitled_%d", i);
        	for (w=WindowList; w!=NULL; w=w->next)
            	    if (!strcmp(w->filename, name))
                	break;
        	    if (w == NULL)
            		break;
    	}

	/* create a new window if neccesary */
	if (window->filenameSet || window->fileChanged)
            window = CreateWindow(name);
		
    	strcpy(window->filename, name);
    	strcpy(window->path, "");
    	window->filenameSet = FALSE;
    	SetWindowModified(window, FALSE);
    	window->readOnly = FALSE;
    	window->lockWrite = FALSE;
    	UpdateWindowReadOnly(window);
    	UpdateStatsLine(window);
		
	BufInsert(window->buffer, 
		  0, 
		  transferInfo->dropData->data.buffers[ii].bp);
    }
}


/*
 * neditTransferCallback -- Called when a dropaction on the area
 *			    set up by neditDropSetup is received.
 */
void neditTransferCallback (Widget	widget,
			    XtPointer 	clientData,
			    XtPointer   callData) {

    DtDndTransferCallbackStruct *transferInfo =
    	(DtDndTransferCallbackStruct *) callData;

    if (transferInfo == NULL) {
	return;
    }

    switch (transferInfo->dropData->protocol) {
    case DtDND_FILENAME_TRANSFER:
 	fileTransferCallback(widget, clientData, callData);
	break;
    case DtDND_BUFFER_TRANSFER:
	dataTransferCallback(widget, clientData, callData);
    	break;
    }
}


/*
 * neditDropSetup -- Registers widget to accept drops of files.
 */
void neditDropSetup (Widget dropDraw) {
    static XtCallbackRec transferCBRec[] =
	{ {neditTransferCallback, NULL},
	  {NULL, NULL} };

    DtDndVaDropRegister(dropDraw,
	    	        DtDND_FILENAME_TRANSFER | DtDND_BUFFER_TRANSFER,
			XmDROP_LINK | XmDROP_COPY,
  	  		transferCBRec,
  	  		DtNtextIsBuffer, True,
  	  		DtNpreserveRegistration, False,
  	  		NULL);
}
